//+------------------------------------------------------------------+
//|                                                    T3 DPO v2.mq4 |
//+------------------------------------------------------------------+

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 LimeGreen
#property indicator_color2 Magenta
#property indicator_width2 2

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    T3Period    = 8;
extern double T3Hot       = 0.7;
extern bool   T3Original  = false;
extern int    UsePrice    = PRICE_CLOSE;
extern bool   Interpolate = true;

//
//
//
//
//

double t3[];
double priceLine[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingDpo;

double alpha;
double c1;
double c2;
double c3;
double c4;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,t3);       SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,priceLine);

   //
   //
   //
   //
   //
   
      double a  = T3Hot;
             c1 = -a*a*a;
             c2 =  3*(a*a+a*a*a);
             c3 = -3*(2*a*a+a+a*a*a);
             c4 = 1+3*a+a*a*a+3*a*a;

      T3Period = MathMax(1,T3Period);
      if (T3Original)
           alpha = 2.0/(1.0 + T3Period);
      else alpha = 2.0/(2.0 + (T3Period-1.0)/2.0);

      //
      //
      //
      //
      //
         
      IndicatorFileName = WindowExpertName();
      CalculatingDpo    = (TimeFrame=="CalculateDpo"); if (CalculatingDpo) return(0);
      ReturningBars     = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
      
   //
   //
   //
   //
   //
         
   IndicatorShortName("T3 dpo "+timeFrameToString(timeFrame)+" ("+T3Period+","+DoubleToStr(T3Hot,2)+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ReturningBars) { t3[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (CalculatingDpo || timeFrame==Period())
   {
      for(i=limit; i>=0; i--)
      {
         t3[i]        = iT3(iMA(NULL,0,T3Period,0,MODE_SMA,UsePrice,i),i,0);
         priceLine[i] =     iMA(NULL,0,1       ,0,MODE_SMA,UsePrice,i);
      }
      return(0);
   }
   
   //
   //
   //
   //
   //
         
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         t3[i]        = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateDpo",T3Period,T3Hot,T3Original,UsePrice,0,y);
         priceLine[i] = iMA(NULL,timeFrame,1,0,MODE_SMA,UsePrice,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
            {
               t3[i+k]        = k*factor*t3[i+n]        + (1.0-k*factor)*t3[i];
               priceLine[i+k] = k*factor*priceLine[i+n] + (1.0-k*factor)*priceLine[i];
            }               
   }
   
   //
   //
   //
   //
   //
   
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double _t3Values[][6];
double iT3(double price,int i,int s)
{
   if (ArrayRange(_t3Values,0) != Bars) ArrayResize(_t3Values,Bars);

   int r = Bars-i-1;
   if (r < 2)
      {
         _t3Values[r][s+0] = price;
         _t3Values[r][s+1] = price;
         _t3Values[r][s+2] = price;
         _t3Values[r][s+3] = price;
         _t3Values[r][s+4] = price;
         _t3Values[r][s+5] = price;
      }
   else
      {
         _t3Values[r][s+0] = _t3Values[r-1][s+0]+alpha*(price            -_t3Values[r-1][s+0]);
         _t3Values[r][s+1] = _t3Values[r-1][s+1]+alpha*(_t3Values[r][s+0]-_t3Values[r-1][s+1]);
         _t3Values[r][s+2] = _t3Values[r-1][s+2]+alpha*(_t3Values[r][s+1]-_t3Values[r-1][s+2]);
         _t3Values[r][s+3] = _t3Values[r-1][s+3]+alpha*(_t3Values[r][s+2]-_t3Values[r-1][s+3]);
         _t3Values[r][s+4] = _t3Values[r-1][s+4]+alpha*(_t3Values[r][s+3]-_t3Values[r-1][s+4]);
         _t3Values[r][s+5] = _t3Values[r-1][s+5]+alpha*(_t3Values[r][s+4]-_t3Values[r-1][s+5]);
      }
   return(c1*_t3Values[r][s+5] + c2*_t3Values[r][s+4] + c3*_t3Values[r][s+3] + c4*_t3Values[r][s+2]);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
string StringUpperCase(string str) { StringToUpper(str); return(str); }